/*
 * TCS Dependency Map Renderer
 *
 * Copyright (c) 2005 TopCoder, Inc. All rights reserved.
 *
 * Author: Pops, TCSDeveloper
 */

using System;
using System.Drawing.Imaging;
using System.Collections;
using NUnit.Framework;

namespace TopCoder.Rules.DependencyMap.View.StressTests
{
    /**
    *   Stress tests Task 3
    */
    [TestFixture]
    public class StressBufferedImageRenderer
    {
        /**
         * default sizes of the buffered image
         */
        private int defaultWidth = 100, defaultHeight = 100;

        /**
         * default type of the buffered image
         */
        private int defaultType = PixelFormat.Format32bppRgb.GetHashCode();
        /**
        * Sample dependency. As per the component specification
        */
        Dependency phoneticSpellCheck;

        /**
        * Dependency purposely built during setup to have
        * a large number of sub-elements
        */
        Dependency myDependency;

        /**
        * number of renderers used for testing
        */
        private const int maxNumRenderers = 100;

        /**
        * number of levels the test dependency tree will have
        */
        private const int maxNumLevels = 4;

        /**
        * number of sub-elements for each level of the test dependency
         * this number is even so as to avoid the blank column issue
        */
        private const int maxNumElements = 11;

        /**
        * List of Diagram renderers
        */
        private IList liste = new ArrayList();

        /**
        * counter for number of dependants
        */
        private int totalCount;

        [SetUp]
        public void setUp()
        {
            //create the sample dependency graph
            Dependency configManager = new Dependency("Config Manager");
            Dependency spellCheck = new Dependency("Spell Check");
            spellCheck.AddDependant(configManager);
            Dependency phoneticPatternMatching = new Dependency("Phonetic Pattern Matching");
            phoneticSpellCheck = new Dependency("Phonetic Spell Check");
            phoneticSpellCheck.AddDependant(configManager);
            phoneticSpellCheck.AddDependant(spellCheck);
            phoneticSpellCheck.AddDependant(phoneticPatternMatching);

            //creates the renderers and add them to the list
            for(int i = 0; i < maxNumRenderers; ++i)
            {
                liste.Add(new BitMapRenderer(PixelFormat.Format32bppRgb));
            }

            //create the large dependency graph
            totalCount = 1;
            myDependency = new Dependency("largeDependency");
            Queue currentLevel = new Queue();
            currentLevel.Enqueue(myDependency);
            for(int i = 0; i < maxNumLevels; ++i)
            {
                Queue nextLevel = new Queue();
                while(currentLevel.Count > 0)
                {
                    Dependency current = (Dependency)currentLevel.Dequeue();
                    for(int j = 0; j < maxNumElements; ++j)
                    {
                        Dependency d = new Dependency("level " + i + " element " + j);
                        current.AddDependant(d);
                        nextLevel.Enqueue(d);
                        ++totalCount;
                    }
                }
                currentLevel = new Queue(nextLevel);
            }
        }

        /**
        * Attempts to render a dependency graph
        * with a large number of renderers
        */
        [Test]
        public void TestManyRenderers()
        {
            // Create the Dependency Renderer and add the diagram renderers
            DependencyRenderer renderer = new DependencyRenderer();
            for(int i = 0; i < maxNumRenderers; ++i)
            {
                renderer.AddDiagramRenderer((IDiagramRenderer)liste[i]);
            }
            DateTime startTime = DateTime.Now;
            // Render the dependency
            renderer.Render(phoneticSpellCheck);
            TimeSpan ts = DateTime.Now - startTime;
            Console.WriteLine("Rendering to " + maxNumRenderers + " renderers took " + ts.TotalMilliseconds + " ms");
        }

        /**
        * Attempts to render a large dependency graph
        */
        [Test]
        public void TestLargeGraph()
        {
            // Create the Dependency Renderer and add a diagram renderer
            DependencyRenderer renderer = new DependencyRenderer();
            renderer.AddDiagramRenderer((IDiagramRenderer)liste[0]);
            DateTime startTime = DateTime.Now;
            // Render the dependency
            renderer.Render(myDependency);
            TimeSpan ts = DateTime.Now - startTime;
            Console.WriteLine("Rendering a graph with " + maxNumLevels + " levels and " + maxNumElements + " elements per level  (total of " + totalCount + " elements) took " + ts.TotalMilliseconds + " ms");
        }
}
}